@@ -1,6 +1,6 @@
 # vim: set fileencoding=utf-8 :
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2011  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2011, 2016  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,14 +15,14 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import absolute_import
 
 import random
 import sys
 import traceback
 
-import gtk
-import gobject
+from gi.repository import Gtk
+from gi.repository import Gdk
+from gi.repository import GObject
 
 from solfege import gu
 from solfege import inputwidgets
@@ -33,7 +33,11 @@ from solfege import cfg, lessonfile, osutils, const
 
 import solfege
 
-class QstatusDefs:
+
+class ExerciseModuleImportError(ImportError):
+    pass
+
+class QstatusDefs(object):
     QSTATUS_NO = 0
     QSTATUS_NEW = 1
     QSTATUS_WRONG = 2
@@ -44,7 +48,9 @@ class QstatusDefs:
     QSTATUS_TYPE_WRONG = 7
     QSTATUS_TYPE_SOLVED = 8
 
+
 class Teacher(cfg.ConfigUtils, QstatusDefs):
+
     def __init__(self, exname):
         cfg.ConfigUtils.__init__(self, exname)
         self.q_status = self.QSTATUS_NO
@@ -62,24 +68,28 @@ class Teacher(cfg.ConfigUtils, QstatusDefs):
         # Some exercises modules want other default values for the
         # lessonfile header variables. They can add them to this dict
         self.m_lessonfile_header_defaults = {}
+
     def maybe_auto_new_question(self):
         if self.get_bool('new_question_automatically'):
             if self.m_timeout_handle is None:
                 def remove_timeout(self=self):
                     self.m_timeout_handle = None
                     self.g_view.new_question()
-                self.m_timeout_handle = gobject.timeout_add(int(self.get_float('seconds_before_new_question')*1000),  remove_timeout)
+                self.m_timeout_handle = GObject.timeout_add(int(self.get_float('seconds_before_new_question') * 1000), remove_timeout)
+
     def end_practise(self):
         if self.m_timeout_handle:
-            gobject.source_remove(self.m_timeout_handle)
+            GObject.source_remove(self.m_timeout_handle)
             self.m_timeout_handle = None
         self.q_status = self.QSTATUS_NO
         soundcard.synth.stop()
+
     def exit_test_mode(self):
         """
         Shared between harmonic and melodic interval.
         """
         self.m_statistics.exit_test_mode()
+
     def set_lessonfile(self, lessonfile):
         """
         Set the variable 'm_lessonfile' and
@@ -89,6 +99,7 @@ class Teacher(cfg.ConfigUtils, QstatusDefs):
         self.parse_lessonfile()
         if self.m_P and self.m_statistics:
             solfege.db.validate_stored_statistics(self.m_P.m_filename)
+
     def parse_lessonfile(self):
         self.m_question = None
         self.q_status = self.QSTATUS_NO
@@ -99,12 +110,14 @@ class Teacher(cfg.ConfigUtils, QstatusDefs):
                                         self.m_lessonfile_header_defaults)
         self.m_P.parse_file(self.m_lessonfile)
         if [q for q in self.m_P.m_questions if isinstance(q.get('music'), lessonfile.Cmdline)]:
-            run = gu.dialog_yesno(_("The lessonfile contain potentially dangerous code because it run external programs. Run anyway?"))
+            run = gu.dialog_yesno(_("The lessonfile contain potentially dangerous code because it run external programs. Run anyway?"), solfege.win)
             if not run:
                 self.m_P = None
+
     def check_askfor(self):
         if self.m_custom_mode:
-            self.set_list('ask_for_names', range(len(self.m_P.get_unique_cnames())))
+            self.set_list('ask_for_names', list(range(len(self.m_P.get_unique_cnames()))))
+
     def play_tonic(self):
         """
         Play the tonic of the question, if defined.
@@ -112,6 +125,7 @@ class Teacher(cfg.ConfigUtils, QstatusDefs):
         if 'tonic' in self.m_P.get_question():
             self.m_P.play_question(None, 'tonic')
 
+
 class MelodicIntervalTeacher(Teacher):
     """
     Base class for interval exercises where
@@ -120,22 +134,25 @@ class MelodicIntervalTeacher(Teacher):
     and singinterval.
     """
     class ConfigureException(Exception):
+
         def __init__(self, msg=_("The exercise has to be better configured.")):
             Exception.__init__(self, msg)
     OK = 0
     ERR_PICKY = 1
     ERR_CONFIGURE = 2
     no_intervals_str = _("No intervals selected for interval number %i.")
+
     def __init__(self, exname):
         Teacher.__init__(self, exname)
         self.m_tonika = None
         self.m_question = []
+
     def new_question(self, L, H):
-        assert isinstance(L, basestring)
-        assert isinstance(H, basestring)
+        assert isinstance(L, str)
+        assert isinstance(H, str)
 
         if self.m_timeout_handle:
-            gobject.source_remove(self.m_timeout_handle)
+            GObject.source_remove(self.m_timeout_handle)
             self.m_timeout_handle = None
 
         if solfege.app.m_test_mode:
@@ -170,7 +187,7 @@ class MelodicIntervalTeacher(Teacher):
             # database (and make it an int if it is not), and nComboBox
             # will make sure the int value is within the correct limits.
             lock_tonic = mpd.MusicalPitch.new_from_int(self.get_int("lock-to-key-note"))
-            lock_scaletype = utils.key_data.keys()[:][self.get_int("lock-to-key-scaletype")]
+            lock_scaletype = list(utils.key_data.keys())[:][self.get_int("lock-to-key-scaletype")]
 
         def try_make_question():
             try:
@@ -182,7 +199,7 @@ class MelodicIntervalTeacher(Teacher):
                 else:
                     self.m_tonika, i = utils.random_tonika_and_interval(L, H,
                             self.get_list('ask_for_intervals_0'))
-            except utils.NoPossibleIntervals, e:
+            except utils.NoPossibleIntervals:
                 raise self.ConfigureException(self.no_intervals_str % 1)
 
             self.m_question = [i]
@@ -208,8 +225,8 @@ class MelodicIntervalTeacher(Teacher):
         for counter in range(max_try):
             try:
                 try_make_question()
-            except self.ConfigureException, e:
-                if counter == max_try -1:
+            except self.ConfigureException:
+                if counter == max_try - 1:
                     raise
                 else:
                     continue
@@ -234,6 +251,7 @@ class MelodicIntervalTeacher(Teacher):
 
 
 class RhythmAddOnClass:
+
     def new_question(self):
         """returns:
                self.ERR_PICKY : if the question is not yet solved and the
@@ -243,7 +261,7 @@ class RhythmAddOnClass:
                self.ERR_NO_ELEMS : if no elements are set to be practised.
         """
         if self.m_timeout_handle:
-            gobject.source_remove(self.m_timeout_handle)
+            GObject.source_remove(self.m_timeout_handle)
             self.m_timeout_handle = None
 
         if self.get_bool('config/picky_on_new_question') \
@@ -268,6 +286,7 @@ class RhythmAddOnClass:
             self.m_question.append(random.choice(v))
         self.q_status = self.QSTATUS_NEW
         return self.OK
+
     def get_music_notenames(self, count_in):
         """
         Return a string with the notenames of the current question.
@@ -282,12 +301,14 @@ class RhythmAddOnClass:
             s = "d%s " % count_in_notelen * self.get_int("count_in")
         s += " ".join([const.RHYTHMS[k] for k in self.m_question])
         return s
+
     def get_music_string(self):
         """
         Return a complete mpd string of the current question that can
         be feed to utils.play_music.
         """
         return r"\rhythmstaff{ \time 1000000/4 %s}" % self.get_music_notenames(True)
+
     def play_rhythm(self, rhythm):
         """
         rhythm is a string. Example: 'c4 c8 c8 c4'
@@ -302,6 +323,7 @@ class RhythmAddOnClass:
         track.replace_note(mpd.notename_to_int("d"),
                            self.get_int("config/countin_perc"))
         soundcard.synth.play_track(track)
+
     def set_elements_variables(self):
         """
         This is called from the on_start_practise() method of exercise
@@ -318,6 +340,7 @@ class RhythmAddOnClass:
                     self.m_P.header.rhythm_elements[:]
                 self.m_P.header.rhythm_elements = \
                   [n for n in self.m_P.header.rhythm_elements if n != 'newline']
+
     def set_default_header_values(self):
         for n, default in (('bpm', 60),
                   ('count_in', 2),
@@ -327,83 +350,88 @@ class RhythmAddOnClass:
             else:
                 self.set_int(n, default)
 
-class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
+
+class Gui(Gtk.VBox, cfg.ConfigUtils, QstatusDefs):
     """Important members:
          - practise_box
          - action_area
-         - config_box
+         - g_config_grid
     """
     short_delay = 700
+
     def __init__(self, teacher, no_notebook=False):
-        gtk.VBox.__init__(self)
+        Gtk.VBox.__init__(self)
         cfg.ConfigUtils.__init__(self, teacher.m_exname)
         assert type(no_notebook) == bool
         self._std_buttons = []
         self.m_key_bindings = {}
         self.m_t = teacher
 
-        vbox = gtk.VBox()
+        vbox = Gtk.VBox()
         vbox.set_spacing(gu.PAD)
         vbox.set_border_width(gu.PAD)
         vbox.show()
 
-        self.practise_box = gtk.VBox()
+        self.practise_box = Gtk.VBox()
         self.practise_box.show()
-        vbox.pack_start(self.practise_box, False)
+        vbox.pack_start(self.practise_box, True, True, 0)
 
-        box = gtk.VBox()
-        self.practise_box.pack_start(box, False, padding=gu.hig.SPACE_LARGE)
+        box = Gtk.VBox()
+        self.practise_box.pack_start(box, False, False, padding=gu.hig.SPACE_LARGE)
         box.show()
-        self.g_lesson_heading = gtk.Label()
-        box.pack_start(self.g_lesson_heading)
+        self.g_lesson_heading = Gtk.Label()
+        box.pack_start(self.g_lesson_heading, True, True, 0)
 
-        self.g_lesson_description = gtk.Label()
+        self.g_lesson_description = Gtk.Label()
         self.g_lesson_description.set_line_wrap(True)
-        box.pack_start(self.g_lesson_description)
+        box.pack_start(self.g_lesson_description, True, True, 0)
 
-        self.action_area = gtk.HBox()
+        self.action_area = Gtk.HBox()
         self.action_area.show()
-        vbox.pack_start(self.action_area, False)
+        vbox.pack_start(self.action_area, False, False, 0)
 
-        self.config_box = gtk.VBox()
-        self.config_box.set_border_width(gu.PAD)
-        self.config_box.show()
-        self.config_box_sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+        self.g_config_grid = Gtk.Grid()
+        self.g_config_grid.set_column_spacing(gu.hig.SPACE_SMALL)
+        self.g_config_grid.set_row_spacing(gu.hig.SPACE_SMALL)
+        self.g_config_grid.props.margin = gu.hig.SPACE_SMALL
         if no_notebook:
-            self.pack_start(vbox)
-            self.pack_start(self.config_box, False)
+            self.pack_start(vbox, False, False, 0)
+            self.pack_start(self.g_config_grid, True, True, 0)
             self.g_notebook = None
         else:
-            self.g_notebook = gtk.Notebook()
-            self.pack_start(self.g_notebook)
+            self.g_notebook = Gtk.Notebook()
+            self.pack_start(self.g_notebook, True, True, 0)
 
-            self.g_notebook.append_page(vbox, gtk.Label(_("Practise")))
-            self.g_notebook.append_page(self.config_box, gtk.Label(_("Config")))
+            self.g_notebook.append_page(vbox, Gtk.Label(label=_("Practise")))
+            self.g_notebook.append_page(self.g_config_grid, Gtk.Label(label=_("Config")))
             self.g_notebook.show()
-        self.g_cancel_test = gtk.Button(_("_Cancel test"))
+        self.g_cancel_test = Gtk.Button(_("_Cancel test"))
         self.g_cancel_test.connect('clicked', self.on_cancel_test)
-        self.action_area.pack_end(self.g_cancel_test, False)
+        self.action_area.pack_end(self.g_cancel_test, False, False, 0)
+
     def add_module_is_deprecated_label(self):
         """
         The deprecated module must set a message in self.g_deprecated_label
         in on_start_practise, preferable telling the file name of the
         lesson file.
         """
-        img = gtk.Image()
-        img.set_from_stock(gtk.STOCK_DIALOG_WARNING,
-                           gtk.ICON_SIZE_BUTTON)
-        hbox = gtk.HBox()
+        img = Gtk.Image()
+        img.set_from_stock(Gtk.STOCK_DIALOG_WARNING,
+                           Gtk.IconSize.BUTTON)
+        hbox = Gtk.HBox()
         hbox.set_border_width(12)
         self.practise_box.set_child_packing(self.g_lesson_heading, False, False, 0, 0)
         hbox.set_spacing(6)
-        hbox.pack_start(img)
-        self.g_deprecated_label = gtk.Label()
-        hbox.pack_start(self.g_deprecated_label)
-        self.practise_box.pack_start(hbox, False, False)
+        hbox.pack_start(img, True, True, 0)
+        self.g_deprecated_label = Gtk.Label()
+        hbox.pack_start(self.g_deprecated_label, True, True, 0)
+        self.practise_box.pack_start(hbox, False, False, 0)
         self.practise_box.reorder_child(hbox, 0)
+
     def set_deprecation_text(self, oldmodule, newmodule, filename):
         self.g_deprecated_label.set_line_wrap(True)
         self.g_deprecated_label.set_markup("<b>The %(oldmodule)s module is deprecated. Convert \"%(filename)s\" to use the %(newmodule)s module.</b>" % locals())
+
     def std_buttons_add(self, *buttons):
         """
         buttons is a sequence of tuples ('buttonname', callback)
@@ -440,6 +468,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
             if '-' in b:
                 b = b.split('-')[0]
             setattr(self, 'g_%s' % b, button)
+
     def _std_buttons_start_sensitivity(self):
         # Values like 'new-interval', 'new-tone' and 'new-chord' and
         # possible others can be added to ._std_buttons
@@ -452,6 +481,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
             'display_music', 'show', 'give_up'):
             if b in self._std_buttons:
                 getattr(self, 'g_%s' % b).set_sensitive(False)
+
     def std_buttons_start_practise(self):
         self._std_buttons_start_sensitivity()
         self.g_new.grab_focus()
@@ -463,7 +493,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         if 'repeat_arpeggio' in self._std_buttons:
             # If one or more of the questions is of musictype 'chord', then
             # we need the "Repeat arpeggio" button.
-            if [q for q in self.m_t.m_P.m_questions \
+            if [q for q in self.m_t.m_P.m_questions
                 if isinstance(q.music, lessonfile.ChordCommon)]:
                 self.g_repeat_arpeggio.show()
             else:
@@ -482,6 +512,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
                 self.g_show.show()
             else:
                 self.g_show.hide()
+
     def std_buttons_new_question(self):
         # Values like 'new-interval', 'new-tone' and 'new-chord' and
         # possible others can be added to ._std_buttons
@@ -500,6 +531,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         for n in 'show', 'give_up', 'guess_answer':
             if n in self._std_buttons:
                 getattr(self, 'g_%s' % n).set_sensitive(False)
+
     def std_buttons_end_practise(self):
         # We check for the lesson parser because it is possible that a
         # completely broken lesson file was started, and the user tries
@@ -514,6 +546,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
                     if '-' in name:
                         name = name.split('-')[0]
                     getattr(self, 'g_%s' % name).hide()
+
     def std_buttons_answer_correct(self):
         # Setting sensitivity is only required when
         # self.get_bool('config/picky_on_new_question') is True, but
@@ -530,9 +563,11 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
             self.g_guess_answer.set_sensitive(False)
         if 'backspace' in self._std_buttons:
             self.g_backspace.set_sensitive(False)
+
     def std_buttons_answer_wrong(self):
         if 'give_up' in self._std_buttons:
             self.g_give_up.set_sensitive(True)
+
     def std_buttons_give_up(self):
         if [n for n in self._std_buttons if n.split("-")[0] == 'new']:
             self.g_new.set_sensitive(True)
@@ -546,12 +581,15 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
             self.g_guess_answer.set_sensitive(False)
         if 'backspace' in self._std_buttons:
             self.g_backspace.set_sensitive(False)
+
     def std_buttons_exception_cleanup(self):
         self._std_buttons_start_sensitivity()
+
     def on_cancel_test(self, *w):
         self.g_cancel_test.hide()
         self.on_end_practise()
         solfege.win.exit_test_mode()
+
     def do_test_complete(self):
         self.on_end_practise()
         req = self.m_t.m_P.get_test_requirement()
@@ -559,9 +597,22 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         solfege.win.exit_test_mode()
         passed, res = solfege.db.get_test_status(self.m_t.m_P.m_filename)
         if res >= req:
-            gu.dialog_ok(_("Test completed!\nYour score was %(score).1f%%.\nThe test requirement was %(requirement).1f%%.") % {'score': res * 100, 'requirement': req * 100})
+            gu.dialog_ok(
+                _("Test completed!\n"
+                  "Your score was %(score).1f%%.\n"
+                  "The test requirement was %(requirement).1f%%.") % {
+                    'score': res * 100,
+                    'requirement': req * 100},
+                solfege.win)
         else:
-            gu.dialog_ok(_("Test failed.\nYour score was %(score).1f%%.\nThe test requirement was %(requirement).1f%%.") % {'score': res * 100, 'requirement': req * 100})
+            gu.dialog_ok(
+                _("Test failed.\n"
+                  "Your score was %(score).1f%%.\n"
+                  "The test requirement was %(requirement).1f%%.") % {
+                    'score': res * 100,
+                    'requirement': req * 100},
+                solfege.win)
+
     def set_lesson_heading(self, txt):
         if txt:
             self.g_lesson_heading.set_text('<span size="large"><b>%s</b></span>' % gu.escape(txt))
@@ -569,6 +620,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
             self.g_lesson_heading.show()
         else:
             self.g_lesson_heading.hide()
+
     def set_lesson_description(self, txt):
         if txt:
             self.g_lesson_description.set_text(gu.escape(txt))
@@ -576,12 +628,13 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
             self.g_lesson_description.show()
         else:
             self.g_lesson_description.hide()
+
     def on_start_practise(self):
         """
         Code that are common for all exercises. Not used by many now,
         but lets see if that can improve.
         """
-        self.handle_config_box_visibility()
+        self.handle_config_grid_visibility()
         self.handle_statistics_page_sensibility()
         if self.get_bool("lock-to-key"):
             self.set_bool("lock-to-key", False)
@@ -595,73 +648,87 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
             self.set_lesson_description(self.m_t.m_P.header.lesson_description)
         else:
             self.set_lesson_description(getattr(self, "lesson_description", None))
+
     def on_end_practise(self):
         pass
-    def handle_config_box_visibility(self):
+
+    def handle_config_grid_visibility(self):
         """
-        Show self.config_box if it has any visible children, otherwise
+        Show self.config_grid if it has any visible children, otherwise
         hide it.
         """
-        if [c for c in self.config_box.get_children() \
+        if [c for c in self.g_config_grid.get_children()
             if c.get_property('visible')]:
-            self.config_box.show()
+            self.g_config_grid.show()
         else:
-            self.config_box.hide()
+            self.g_config_grid.hide()
+
     def handle_statistics_page_sensibility(self):
         try:
             if self.m_t.m_custom_mode:
                 self.g_statview.set_sensitive(False)
-                self.g_statview.set_tooltip_text(_(u"Statistics is disabled. Either because you selected a “Configure yourself” exercise, or because “Expert mode” is selected in the preferences window."))
+                self.g_statview.set_tooltip_text(_("Statistics is disabled. Either because you selected a “Configure yourself” exercise, or because “Expert mode” is selected in the preferences window."))
             else:
                 self.g_statview.set_sensitive(True)
                 self.g_statview.set_tooltip_text("")
-        except AttributeError: # not all exercises has g_statview
+        except AttributeError:  # not all exercises has g_statview
             pass
+
     def on_key_press_event(self, widget, event):
         if (self.g_notebook is None or self.g_notebook.get_current_page() == 0) \
-           and event.type == gtk.gdk.KEY_PRESS:
+                and event.type == Gdk.EventType.KEY_PRESS:
             for s in self.m_key_bindings:
                 if self.keymatch(event, s):
                     self.m_key_bindings[s]()
                     return 1
+
     def keymatch(self, event, cfname):
         a, b = gu.parse_key_string(self.get_string(cfname))
-        return ((event.state & (gtk.gdk.CONTROL_MASK|gtk.gdk.SHIFT_MASK|gtk.gdk.MOD1_MASK)) == a) and (event.keyval == b)
+        return ((event.get_state() & (Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK | Gdk.ModifierType.MOD1_MASK)) == a) and (event.keyval == b)
+
     def setup_statisticsviewer(self, viewclass, heading):
         self.g_statview = viewclass(self.m_t.m_statistics, heading)
         self.g_statview.show()
-        self.g_notebook.append_page(self.g_statview, gtk.Label(_("Statistics")))
+        self.g_notebook.append_page(self.g_statview, Gtk.Label(label=_("Statistics")))
         self.g_notebook.connect('switch_page', self.on_switch_page)
+
     def on_switch_page(self, notebook, obj, pagenum):
         if pagenum == 2:
             if self.m_t.m_P and not self.m_t.m_custom_mode:
                 self.g_statview.update()
             else:
                 self.g_statview.clear()
-    def _add_auto_new_question_gui(self, box):
-        hbox = gu.bHBox(box, False)
-        hbox.set_spacing(gu.PAD_SMALL)
-        adj = gtk.Adjustment(0, 0, 10, 0.1, 1)
+
+    def _add_auto_new_question_gui(self, row):
+        """
+        Add the user interface used to configure if we should get a new
+        question automatically. The GUI is added to self.g_config_grid
+        at the row set by the ROW variable.
+        """
+        adj = Gtk.Adjustment(0, 0, 10, 0.1, 1)
         spin = gu.nSpinButton(self.m_exname, 'seconds_before_new_question',
                        adj)
         spin.set_digits(1)
-        label = gtk.Label(_("Delay (seconds):"))
-        label.show()
+        label = Gtk.Label(label=_("seconds delay"))
+        label.props.halign = Gtk.Align.START
+
         def f(button, spin=spin, label=label):
             spin.set_sensitive(button.get_active())
             label.set_sensitive(button.get_active())
         b = gu.nCheckButton(self.m_exname, 'new_question_automatically',
-                            _("_New question automatically."), callback=f)
-        hbox.pack_start(b, False)
+                            _("_New question automatically"), callback=f)
+        b.props.halign = Gtk.Align.END
         label.set_sensitive(b.get_active())
-        hbox.pack_start(label, False)
         spin.set_sensitive(b.get_active())
-        hbox.pack_start(spin, False)
+        self.g_config_grid.attach(b, 0, row, 1, 1)
+        self.g_config_grid.attach(spin, 1, row, 1, 1)
+        self.g_config_grid.attach(label, 2, row, 1, 1)
+
     def _lessonfile_exception(self, exception, sourcefile, lineno):
         m = gu.ExceptionDialog(exception)
         idx = self.m_t.m_P._idx
         if idx is not None:
-            m.add_text(_('Please check question number %(idx)i in the lesson file "%(lf)s".') % {'idx': self.m_t.m_P._idx+1, 'lf': self.m_t.m_P.m_filename})
+            m.add_text(_('Please check question number %(idx)i in the lesson file "%(lf)s".') % {'idx': self.m_t.m_P._idx + 1, 'lf': self.m_t.m_P.m_filename})
         if 'm_nonwrapped_text' in dir(exception):
             m.add_nonwrapped_text(exception.m_nonwrapped_text)
         if 'm_mpd_badcode' in dir(exception) and exception.m_mpd_badcode:
@@ -671,6 +738,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         m.add_text(_('The exception was caught in\n"%(filename)s", line %(lineno)i.') % {'filename': sourcefile, 'lineno': lineno})
         m.run()
         m.destroy()
+
     def _mpd_exception(self, exception, sourcefile, lineno):
         m = gu.ExceptionDialog(exception)
         if 'm_mpd_varname' in dir(exception):
@@ -685,15 +753,17 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         m.add_text(_('The exception was caught in\n"%(filename)s", line %(lineno)i.') % {'filename': sourcefile, 'lineno': lineno})
         m.run()
         m.destroy()
+
     def run_exception_handled(self, method, *args, **kwargs):
         """
         Call method() and catch exceptions with standard_exception_handler.
         """
         try:
             return method(*args, **kwargs)
-        except Exception, e:
+        except Exception as e:
             if not self.standard_exception_handler(e):
                 raise
+
     def standard_exception_handler(self, e, cleanup_function=lambda: False):
         """
         Use this method to try to catch a few common solfege exceptions.
@@ -710,9 +780,6 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
                 raise
         """
         sourcefile, lineno, func, code = traceback.extract_tb(sys.exc_info()[2])[0]
-        # We can replace characters because we will only display the
-        # file name, not open the file.
-        sourcefile = sourcefile.decode(sys.getfilesystemencoding(), 'replace')
         if solfege.app.m_options.disable_exception_handler:
             return False
         elif isinstance(e, lessonfile.NoQuestionsConfiguredException):
@@ -737,43 +804,41 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
             return True
         return False
 
+
 class RhythmAddOnGuiClass(object):
-    def add_select_elements_gui(self):
-        self.g_element_frame = frame = gtk.Frame(_("Rhythms to use in question"))
-        self.config_box.pack_start(frame, False)
+
+    def add_select_elements_gui(self, grid, row):
+        self.g_element_frame = frame = Gtk.Frame(label=_("Rhythms to use in question"))
+        grid.attach(frame, 0, row, 3, 1)
         self.g_select_rhythms_box = gu.NewLineBox()
         self.g_select_rhythms_box.set_border_width(gu.PAD_SMALL)
         frame.add(self.g_select_rhythms_box)
-    def add_select_num_beats_gui(self):
+
+    def add_select_num_beats_gui(self, grid, row):
         ###
-        hbox = gtk.HBox()
-        hbox.set_spacing(gu.hig.SPACE_SMALL)
-        label = gtk.Label(_("Number of beats in question:"))
-        hbox.pack_start(label, False)
-        self.config_box_sizegroup.add_widget(label)
-        label.set_alignment(1.0, 0.5)
-        hbox.pack_start(gu.nSpinButton(self.m_exname, "num_beats",
-                     gtk.Adjustment(4, 1, 100, 1, 10)), False)
-        self.config_box.pack_start(hbox, False)
-        hbox.show_all()
+        label = Gtk.Label(label=_("Number of beats in question:"))
+        label.props.halign = Gtk.Align.END
+        label.show()
+        grid.attach(label, 0, row, 1, 1)
+        grid.attach(gu.nSpinButton(self.m_exname, "num_beats",
+                                   Gtk.Adjustment(4, 1, 100, 1, 10)),
+                    1, row, 1, 1)
         #
-        hbox = gtk.HBox()
-        hbox.set_spacing(gu.hig.SPACE_SMALL)
-        label = gtk.Label(_("Count in before question:"))
-        hbox.pack_start(label, False)
-        self.config_box_sizegroup.add_widget(label)
-        label.set_alignment(1.0, 0.5)
-        hbox.pack_start(label, False)
-        hbox.pack_start(gu.nSpinButton(self.m_exname, "count_in",
-                     gtk.Adjustment(2, 0, 10, 1, 10)), False)
-        hbox.show_all()
-        self.config_box.pack_start(hbox, False)
+        label = Gtk.Label(label=_("Count in before question:"))
+        label.props.halign = Gtk.Align.END
+        label.show()
+        grid.attach(label, 0, row + 1, 1, 1)
+        grid.attach(gu.nSpinButton(self.m_exname, "count_in",
+                                   Gtk.Adjustment(2, 0, 10, 1, 10)),
+                    1, row + 1, 1, 1)
+
     def pngcheckbutton(self, i):
-        btn = gtk.CheckButton()
+        btn = Gtk.CheckButton()
         btn.add(gu.create_rhythm_image(const.RHYTHMS[i]))
         btn.show()
         btn.connect('clicked', self.select_element_cb, i)
         return btn
+
     def update_select_elements_buttons(self):
         """
         (Re)create the checkbuttons used to select which rhythm elements
@@ -789,6 +854,7 @@ class RhythmAddOnGuiClass(object):
                 self.g_select_rhythms_box.add_widget(b)
                 b.set_active(n in self.m_t.m_P.header.rhythm_elements)
         self.g_select_rhythms_box.show_widgets()
+
     def select_element_cb(self, button, element_num):
         def sortlike(orig, b):
             ret = []
@@ -812,11 +878,13 @@ class RhythmAddOnGuiClass(object):
         self.m_t.m_P.header.rhythm_elements = \
             [n for n in self.m_t.m_P.header.rhythm_elements if n != 'newline']
 
+
 class IntervalGui(Gui):
     """
     Creates 'New interval' and 'Repeat' buttons in the action_area.
     """
     keyboard_accel = 99
+
     def __init__(self, teacher):
         Gui.__init__(self, teacher)
 
@@ -824,22 +892,21 @@ class IntervalGui(Gui):
 
         self.g_flashbar = gu.FlashBar()
         self.g_flashbar.show()
-        self.practise_box.pack_start(self.g_flashbar, False)
+        self.practise_box.pack_start(self.g_flashbar, False, False, 0)
         self.practise_box.set_spacing(gu.PAD)
 
         self.std_buttons_add(('new-interval', self.new_question),
             ('repeat', self.repeat_question))
         self.setup_key_bindings()
-    def _create_select_inputwidget_gui(self):
+
+    def _create_select_inputwidget_gui(self, row):
         """
         This will be called by HarmonicInterval and MelodicInterval
         constructor
         """
-        hbox = gu.bHBox(self.config_box, False)
-        hbox.set_spacing(gu.PAD_SMALL)
-        gu.bLabel(hbox, _("Input interface:"), False)
-
-        combo = gtk.combo_box_new_text()
+        label = Gtk.Label(_("Input interface"))
+        label.props.halign = Gtk.Align.END
+        combo = Gtk.ComboBoxText()
         for i in range(len(inputwidgets.inputwidget_names)):
             combo.append_text(inputwidgets.inputwidget_names[i])
         if self.get_int('inputwidget') < len(inputwidgets.inputwidget_names):
@@ -847,31 +914,33 @@ class IntervalGui(Gui):
         else:
             combo.set_active(0)
         combo.connect('changed', lambda w: self.use_inputwidget(w.get_active()))
-        hbox.pack_start(combo, False)
 
         self.g_disable_unused_buttons = gu.nCheckButton(self.m_exname,
                     'disable_unused_intervals', _("_Disable unused buttons"))
-        hbox.pack_start(self.g_disable_unused_buttons)
-    def add_lock_to_key_gui(self):
+        self.g_config_grid.attach(label, 0, row, 1, 1)
+        self.g_config_grid.attach(combo, 1, row, 1, 1)
+        self.g_config_grid.attach(self.g_disable_unused_buttons, 2, row, 1, 1)
+
+    def add_lock_to_key_gui(self, row):
         # gui to lock to a key
         def toggle_lock_to_key_sensitivity(checkbutton):
             self.g_notename.set_sensitive(checkbutton.get_active())
             self.g_scaletype.set_sensitive(checkbutton.get_active())
-        self.g_lock_to_key_hbox = gtk.HBox()
-        self.config_box.pack_start(self.g_lock_to_key_hbox, False)
         check = gu.nCheckButton(self.m_exname, 'lock-to-key',
-            _("Lock intervals to key:"),
-            callback = toggle_lock_to_key_sensitivity)
-        self.g_lock_to_key_hbox.pack_start(check, False)
+            _("Lock intervals to key"),
+            callback=toggle_lock_to_key_sensitivity)
+        check.props.halign = Gtk.Align.END
         self.g_notename = gu.nComboBox(self.m_exname, 'lock-to-key-note',
             mpd.MusicalPitch.new_from_int(60).get_user_notename(),
-            [mpd.MusicalPitch.new_from_int(60 + x).get_user_notename() for x in range(12)])
+            [mpd.MusicalPitch.new_from_int(60 + i).get_user_notename() for i in range(12)])
         self.g_notename.show()
-        self.g_lock_to_key_hbox.pack_start(self.g_notename, False)
-        self.g_scaletype = gu.nComboBox(self.m_exname, 'lock-to-key-scaletype', _("Major"), [n['name'] for n in utils.key_data.values()])
+        self.g_scaletype = gu.nComboBox(self.m_exname, 'lock-to-key-scaletype', _("Major"), [n['name'] for n in list(utils.key_data.values())])
         self.g_scaletype.show()
-        self.g_lock_to_key_hbox.pack_start(self.g_scaletype, False)
+        self.g_config_grid.attach(check, 0, row, 1, 1)
+        self.g_config_grid.attach(self.g_notename, 1, row, 1, 1)
+        self.g_config_grid.attach(self.g_scaletype, 2, row, 1, 1)
         toggle_lock_to_key_sensitivity(check)
+
     def select_inputwidget(self):
         """
         This will be called by HarmonicInterval and MelodicInterval
@@ -881,6 +950,7 @@ class IntervalGui(Gui):
         if i >= len(inputwidgets.inputwidget_names):
             i = 0
         self.use_inputwidget(i)
+
     def use_inputwidget(self, i):
         self.set_int('inputwidget', i)
         if self.g_input:
@@ -896,7 +966,7 @@ class IntervalGui(Gui):
         self.g_input = inputwidgets.int_to_inputwidget(
             i, self.click_on_interval, self.m_exname,
             self.get_interval_input_list, v)
-        self.practise_box.pack_start(self.g_input)
+        self.practise_box.pack_start(self.g_input, True, True, 0)
         self.practise_box.reorder_child(self.g_input, 1)
         self.g_input.show()
         if self.m_t.m_tonika:
@@ -905,6 +975,7 @@ class IntervalGui(Gui):
             self.on_end_practise()
         self.g_disable_unused_buttons.set_sensitive(
             isinstance(self.g_input, inputwidgets.IntervalButtonsWidgetBase))
+
     def setup_key_bindings(self):
         keys = ['minor2', 'major2', 'minor3', 'major3',
                 'perfect4', 'diminished5', 'perfect5', 'minor6',
@@ -912,33 +983,41 @@ class IntervalGui(Gui):
                 'minor9', 'major9', 'minor10', 'major10']
         self.m_key_bindings = {}
         for idx in range(len(keys)):
-            self.m_key_bindings['interval_input/'+keys[idx]] = lambda idx=idx, self=self: self.click_on_interval(self.keyboard_accel, idx+1, None)
+            self.m_key_bindings['interval_input/' + keys[idx]] = lambda idx=idx, self=self: self.click_on_interval(self.keyboard_accel, idx + 1, None)
+
     def repeat_question(self, *w):
         self.m_t.play_question()
         self.g_input.grab_focus_first_sensitive_button()
 
 
 class LessonbasedGui(Gui):
+
     def __init__(self, teacher, no_notebook=False):
         Gui.__init__(self, teacher, no_notebook)
-    def add_random_transpose_gui(self):
-        self.g_random_transpose_box = hbox = gu.bHBox(self.config_box, False, False)
-        label = gtk.Label(_("Random transpose:"))
-        label.show()
-        hbox.pack_start(label, False)
-        hbox.set_spacing(6)
-        self.g_random_transpose = gtk.Label()
-        self.g_random_transpose.show()
-        hbox.pack_start(self.g_random_transpose)
 
-        button = gtk.Button(_("Change ..."))
+    def add_random_transpose_gui(self, row):
+        """
+        All widgets need an instance variable because we will hide
+        and show the widgets.
+        """
+        ###
+        self.g_random_transpose_label = label = Gtk.Label(label=_("Random transpose:"))
+        label.show()
+        self.g_config_grid.attach(label, 0, row, 1, 1)
+        ###
+        self.g_random_transpose = Gtk.Label()
+        self.g_random_transpose.show()
+        self.g_config_grid.attach(self.g_random_transpose, 1, row, 1, 1)
+        ###
+        self.g_random_transpose_button = button = Gtk.Button(_("Change ..."))
         button.show()
         button.connect('clicked', self.run_random_transpose_dialog)
-        hbox.pack_start(button)
+        self.g_config_grid.attach(button, 2, row, 1, 1)
+
     def run_random_transpose_dialog(self, widget):
         dlg = RandomTransposeDialog(self.m_t.m_P.header.random_transpose, solfege.win)
         response = dlg.run()
-        if response == gtk.RESPONSE_OK:
+        if response == Gtk.ResponseType.OK:
             self.m_t.m_P.header.random_transpose = dlg.get_value()
             if self.m_t.m_P.header.random_transpose[0] == True:
                 self.g_random_transpose.set_text(_("Yes"))
@@ -947,6 +1026,7 @@ class LessonbasedGui(Gui):
             else:
                 self.g_random_transpose.set_text(str(self.m_t.m_P.header.random_transpose))
         dlg.destroy()
+
     def show_answer(self, widget=None):
         """
         Show the answer in the g_music_displayer if we have one, if not
@@ -959,6 +1039,7 @@ class LessonbasedGui(Gui):
         if not isinstance(self.m_t.m_P.get_question()[varname], lessonfile.MpdDisplayable):
             return
         self.display_music(varname)
+
     def display_music(self, varname):
         """
         Display the music in the variable named by varname from
@@ -971,12 +1052,13 @@ class LessonbasedGui(Gui):
                 self.g_music_displayer.display(self.m_t.m_P.get_music(varname), fontsize)
             else:
                 solfege.win.display_in_musicviewer(self.m_t.m_P.get_music(varname))
-        except mpd.MpdException, e:
+        except mpd.MpdException as e:
             self.m_t.m_P.get_question()[varname].complete_to_musicdata_coords(self.m_t.m_P, e)
             e.m_mpd_varname = varname
             e.m_mpd_badcode = self.m_t.m_P.get_question()[varname].get_err_context(e, self.m_t.m_P)
             if not self.standard_exception_handler(e):
                 raise
+
     def do_at_question_start_show_play(self):
         """
         This method is shared by idbyname and elembuilder, and possibly
@@ -998,6 +1080,7 @@ class LessonbasedGui(Gui):
                 self.show_answer()
             elif 'cuemusic' in self.m_t.m_P.get_question():
                 self.display_music('cuemusic')
+
     def show_hide_at_question_start_buttons(self):
         """
         Show and hide g_play_music, g_repeat and g_display_music
@@ -1014,4 +1097,3 @@ class LessonbasedGui(Gui):
             self.g_display_music.show()
         else:
             self.g_display_music.hide()
-
